/* (C) 2021 YiRing, Inc. */
package com.yiring.common.swagger;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.lang.Console;
import cn.hutool.core.net.NetUtil;
import com.yiring.common.core.I18n;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.models.ExternalDocumentation;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.info.Contact;
import io.swagger.v3.oas.models.info.Info;
import io.swagger.v3.oas.models.tags.Tag;
import java.util.*;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.customizers.OpenApiCustomizer;
import org.springdoc.core.models.GroupedOpenApi;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;

/**
 * Swagger Config
 *
 * @author ifzm
 * @version 0.1
 * 2019/3/11 19:13
 */

@Slf4j
@Profile("!prod")
@Configuration
@RequiredArgsConstructor
public class SwaggerConfig implements CommandLineRunner {

    @Value("${spring.application.name}")
    String applicationName;

    @Value("${server.port}")
    String port;

    @Value("${server.servlet.context-path}")
    String path;

    final I18n i18n;

    @Bean
    public OpenAPI openAPI() {
        Info info = new Info()
            .title("API Doc")
            .description(applicationName)
            .version("1.0")
            .contact(new Contact().name("© YiRing").url("https://yiring.com").email("developer@yiring.com"));

        return new OpenAPI().info(info).externalDocs(new ExternalDocumentation());
    }

    @Bean(name = "api.any")
    public GroupedOpenApi any() {
        return api("default", List.of("com.yiring"), List.of("/**"), Collections.emptyList());
    }

    @Bean(name = "api.auth")
    public GroupedOpenApi auth() {
        return api("① Auth", List.of("com.yiring.auth.web"), List.of("/**"), List.of("/sys/**"));
    }

    @Bean(name = "api.common")
    public GroupedOpenApi common() {
        return api(
            "② 公共",
            List.of("com.yiring.common.web", "com.yiring.app.web.common"),
            List.of("/**"),
            Collections.emptyList()
        );
    }

    @Bean(name = "api.manage")
    public GroupedOpenApi manage() {
        return api("③ 系统管理", List.of("com.yiring.auth.web.sys"), List.of("/**"), Collections.emptyList());
    }

    @Bean(name = "api.example")
    public GroupedOpenApi example() {
        return api("④ 示例", List.of("com.yiring.app.web.example"), List.of("/**"), Collections.emptyList());
    }

    @Bean
    public OpenApiCustomizer sortTagCustom() {
        String order = "x-order";
        return api -> {
            List<Tag> tags = api
                .getTags()
                .stream()
                .sorted(
                    Comparator.comparing(tag ->
                        Convert.toInt(
                            Optional.ofNullable(tag.getExtensions()).orElseGet(HashMap::new).get(order),
                            Integer.MAX_VALUE
                        )
                    )
                )
                .peek(tag -> {
                    Map<String, Object> extensions = tag.getExtensions();
                    if (Objects.nonNull(extensions) && extensions.containsKey(order)) {
                        extensions.put(order, Convert.toInt(extensions.get(order)));
                    }
                })
                .toList();
            api.setTags(tags);
        };
    }

    //    @Bean
    //    public OperationCustomizer addGlobalStatusResponse() {
    //        return (operation, handlerMethod) -> {
    //            ApiResponses responses = operation.getResponses();
    //            for (Status status : Status.values()) {
    //                if (Status.OK.equals(status)) {
    //                    continue;
    //                }
    //
    //                ApiResponse response = new ApiResponse()
    //                    .description(i18n.get(status.getReasonPhrase()))
    //                    .$ref("Response");
    //                responses.addApiResponse(String.valueOf(status.value()), response);
    //            }
    //
    //            return operation;
    //        };
    //    }

    private GroupedOpenApi api(
        String group,
        List<String> basePackages,
        List<String> pathsToMatch,
        List<String> pathsToExclude
    ) {
        return GroupedOpenApi
            .builder()
            .group(group)
            .packagesToScan(basePackages.toArray(new String[0]))
            .addOpenApiMethodFilter(method -> method.isAnnotationPresent(Operation.class))
            .pathsToMatch(pathsToMatch.toArray(new String[0]))
            .pathsToExclude(pathsToExclude.toArray(new String[0]))
            .build();
    }

    @Override
    public void run(String... args) {
        String link = NetUtil
            .localIpv4s()
            .stream()
            .map(host -> "> http://" + host + ":" + port + path + "/doc.html")
            .collect(Collectors.joining("\n\t\t"));
        Console.log("\n\t📖 API Doc (OpenAPI3): \n\t\t{}\n", link);
    }
}
